package com.xdf.creative.module.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.WebInitiationApplyQueryParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.WebInitiationApplyQueryVo;
import com.xdf.creative.enums.InitiationApplryAduitStateEnum;
import com.xdf.creative.enums.InitiationApplryAduitTypeEnum;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.module.entity.WebInitiationApply;
import com.xdf.creative.module.mapper.WebInitiationApplyMapper;
import com.xdf.creative.module.service.WebInitiationApplyService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Date;


/**
 * <p>
 * 文促会入会申请表 服务实现类
 * </p>
 *
 * @author DeanYe
 * @since 2019-10-30
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class WebInitiationApplyServiceImpl extends BaseServiceImpl<WebInitiationApplyMapper, WebInitiationApply> implements WebInitiationApplyService {

    @Autowired
    private WebInitiationApplyMapper webInitiationApplyMapper;

    @Override
    public WebInitiationApplyQueryVo getWebInitiationApplyById(Serializable id) throws Exception {
        return webInitiationApplyMapper.getWebInitiationApplyById(id);
    }

    /**
     * 后台管理获取分页对象
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getWebInitiationApplyPageListBack(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getWebInitiationApplyPageListBack(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    /**
     * web首页获取分页对象
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getWebInitiationApplyPageListHome(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getWebInitiationApplyPageListHome(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    /**
     * 获取初始审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByFirstAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByFirstAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 获取信用审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByCreditAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_PASS.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByCreditAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 获取最终审核分页
     *
     * @param webInitiationApplyQueryParam
     * @return
     * @throws Exception
     */
    @Override
    public PageTool<WebInitiationApplyQueryVo> getPageListByFinalAudit(WebInitiationApplyQueryParam webInitiationApplyQueryParam) throws Exception {
        Page page = setPageParam(webInitiationApplyQueryParam);
        webInitiationApplyQueryParam.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_PASS.getCode());
        IPage<WebInitiationApplyQueryVo> iPage = webInitiationApplyMapper.getPageListByFinalAudit(page, webInitiationApplyQueryParam);
        return new PageTool(iPage);
    }

    ;


    /**
     * 用户提交审核确认
     *
     * @param webInitiationApply
     * @return
     */
    public ApiResult<Boolean> sendAudit(WebInitiationApply webInitiationApply) {
        if (StringUtils.isEmpty(webInitiationApply.getIds())) {
            return ApiResult.error(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }
        if (null == webInitiationApply.getAuditType()) {
            return ApiResult.error(StatusCode.WEB_INITIATION_APPLY_FIRST_AUDIT);
        }
        String[] ids = webInitiationApply.getIds().split(";");
        int successNum = ids.length;
        int failedTotal = 0;
        if (successNum <= 0) {
            return ApiResult.error(StatusCode.WEB_INITIATION_APPLY_SEND_AUDIT);
        }
        boolean successflag = true;
        for (int i = 0; i < successNum; i++) {
            WebInitiationApply webInitiationApplyNew = this.getById(ids[i]);
            if (null == webInitiationApplyNew) {
                failedTotal++;
                successflag = false;
                ;
            } else {
                if (InitiationApplryAduitTypeEnum.INITIATION_APPLRY_ADUIT_TYPE_INIT.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setFirstAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setFirstSubmitTime(new Date());
                    webInitiationApplyNew.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_WAIT.getCode());
                } else if (InitiationApplryAduitTypeEnum.INITIATION_APPLRY_ADUIT_TYPE_SECOND.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setCreditAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setCreditSubmitTime(new Date());
                    webInitiationApplyNew.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_WAIT.getCode());
                } else if (InitiationApplryAduitTypeEnum.NITIATION_APPLRY_ADUIT_TYPE_FINAL.getCode().equals(webInitiationApply.getAuditType())) {
                    webInitiationApplyNew.setFinalAudit(InitiationApplryAduitStateEnum.INITIATION_APPLRY_ADUIT_STATE_SEND.getCode());
                    // webInitiationApplyNew.setFinalSubmitTime(new Date());
                }
                webInitiationApplyNew.setUpdateTime(new Date());
                boolean flag = this.updateById(webInitiationApplyNew);
                if (!flag) {
                    failedTotal++;
                    successflag = false;
                }
            }
        }
        if (successflag) {
            return ApiResult.result(successflag);
        } else {
            return ApiResult.fail(String.format(StatusCode.WEB_INITIATION_APPLY_SEND_SUCCESS.getMsg(), failedTotal));
        }
    }

    ;

}
